# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import nbformat
import unittest

from . import open_in_colab

def md_cell(source, id=''):
  return nbformat.v4.new_markdown_cell(source, metadata={'id': id})


def code_cell(source, id=''):
  return nbformat.v4.new_code_cell(source, metadata={'id': id})


class OpenInColabTest(unittest.TestCase):
  def test_open_in_colab(self):
    expected = [
        md_cell(
            '<a href="https://colab.research.google.com/github/user/repo/blob/master/notebook.ipynb" target="_parent">'
              '<img src="https://colab.research.google.com/assets/colab-badge.svg" alt="Open in Colab"/>'
            '</a>',
            id='view-in-github',
        ),
        md_cell('content', id='H1'),
    ]
    actual = list(open_in_colab(
        cells=[md_cell('content', id='H1')],
        ipynb_github_url='https://github.com/user/repo/blob/master/notebook.ipynb',
    ))
    self.assertEqual(expected, actual)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import jinja2
import json
import tempfile
import unittest

from unittest.mock import patch

from . import GithubSampleExt
from . import MarkdownLoader

expected_file = 'test/hello-expected.md'
variables_file = 'test/hello-variables.json'

with open(expected_file) as f:
  expected = f.read().rstrip()
with open(variables_file) as f:
  variables = json.load(f)

env = jinja2.Environment(loader=MarkdownLoader(), extensions=[GithubSampleExt])


def render_string(source, variables=None, env=env):
  with tempfile.NamedTemporaryFile('w') as f:
    f.write(source)
    f.seek(0)
    template = env.get_template(f.name)
  return template.render(variables or {})

class MarkdownLoaderTest(unittest.TestCase):
  def test_from_file_markdown(self):
    template = env.get_template('test/hello.md')
    actual = template.render(variables)
    self.assertEqual(expected, actual)

  def test_from_file_html(self):
    with open('test/hello-html-expected.md') as f:
      expected = f.read().strip()
    template = env.get_template('test/hello.html')
    actual = template.render(variables)
    self.assertEqual(expected, actual)

  def test_include(self):
    actual = render_string('\n'.join([
        "{% include 'test/title.md' %}",
        '',
        'Hello {{name}}!',
        '',
        '```py',
        '{% github_sample /davidcavazos/md2ipynb/blob/master/examples/code/hello-world.py tag:hello_world %}',
        '```',
    ]), variables)
    self.assertEqual(expected, actual)

  def test_include_searchpath(self):
    env = jinja2.Environment(loader=MarkdownLoader('test/'),
                             extensions=[GithubSampleExt])
    actual = render_string('\n'.join([
        "{% include 'title.md' %}",
        '',
        'Hello {{name}}!',
        '',
        '```py',
        '{% github_sample /davidcavazos/md2ipynb/blob/master/examples/code/hello-world.py tag:hello_world %}',
        '```',
    ]), variables, env)
    self.assertEqual(expected, actual)

  def test_include_not_found(self):
    template = env.from_string("{% include 'non-existent-file.md' %}")
    with self.assertRaises(jinja2.exceptions.TemplateNotFound):
      template.render()

  def test_code_block(self):
    expected = '\n'.join([
        '```',
        "# Hello world in Python.",
        "print('Hello from Python!')",
        '```',
    ])
    actual = render_string('\n'.join([
        '```',
        '{% github_sample /davidcavazos/md2ipynb/blob/master/examples/code/hello-world.py tag:hello_world %}',
        '```',
    ]))
    self.assertEqual(expected, actual)

  def test_code_block_ending_inline(self):
    expected = '\n'.join([
        '```',
        "# Hello world in Python.",
        "print('Hello from Python!')",
        '```',
    ])
    actual = render_string('\n'.join([
        '```',
        '{% github_sample /davidcavazos/md2ipynb/blob/master/examples/code/hello-world.py tag:hello_world %}```',
    ]))
    self.assertEqual(expected, actual)

  def test_jekyll_liquid_include_static(self):
    expected = '\n'.join([
        '# Include static',
        '# Include static',
    ])
    actual = render_string('\n'.join([
        '{% include test/include-static.md %}',
        '{% include "test/include-static.md" %}',
    ]))
    self.assertEqual(expected, actual)

  def test_jekyll_liquid_include_argument(self):
    expected = '\n'.join([
        '# Include argument',
        '# Include argument',
    ])
    actual = render_string('\n'.join([
        '{% include test/include-argument.md title="Include argument" %}',
        '{% include "test/include-argument.md" title="Include argument" %}',
    ]))
    self.assertEqual(expected, actual)

  def test_jekyll_liquid_include_argument_variable(self):
    expected = '\n'.join([
        '',
        '# Include argument variable',
        '# Include argument variable',
    ])
    actual = render_string('\n'.join([
        '{% capture var %}Include argument variable{% endcapture %}',
        '{% include test/include-argument.md title=var %}',
        '{% include "test/include-argument.md" title=var %}',
    ]))
    self.assertEqual(expected, actual)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import jinja2

from md2ipynb import read


def imports(sections, imports=None, variables=None, include_dir=None, jinja_env=None):
  def sections_from_imports(import_index):
    for input_file in imports[import_index]:
      for import_section in read.sections(input_file, variables, include_dir, jinja_env):
        yield import_section

  sections = list(sections)
  if imports is None:
    imports = {}

  # Normalize imports to the form: `{non_negative_index: [file1, file2, ...]}`.
  for index, input_file in imports.items():
    if index < 0:
      imports[len(sections)+index+1] = input_file
      del imports[index]

  # Iterate over all the sections, inserting any imports if needed.
  for i, section in enumerate(sections):
    if i in imports:
      for import_section in sections_from_imports(i):
        yield import_section
    yield section

  # Include imports that go at the end.
  i = len(sections)
  if i in imports:
    for import_section in sections_from_imports(i):
      yield import_section



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from . import paragraphs


def sections(input_file='-', variables=None, include_dir=None, jinja_env=None, start_on_header=True):
  header_found = False
  section = []
  for paragraph in paragraphs(input_file, variables, jinja_env):
    if paragraph[0].startswith('#'):
      if section and (not start_on_header or header_found):
        yield section
      section = []
      header_found = True
    section.append(paragraph)
  if section:
    yield section



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import html2md
import jinja2
import os
import re


class MarkdownLoader(jinja2.BaseLoader):
  def __init__(self, searchpath='.'):
    self.searchpath = searchpath

  def get_source(self, env, name):
    path = os.path.join(self.searchpath, name)
    if not os.path.exists(path):
      raise jinja2.TemplateNotFound(path)

    mtime = os.path.getmtime(path)
    with open(path) as f:
      source = html2md.convert(f.read())

    # Normalize source code before applying any templating logic.
    source = normalize_inline_code_block(source)
    source = jekyll_liquid_capture(source)
    source = jekyll_liquid_include(source)
    return source, path, lambda: mtime == os.path.getmtime(path)

  
def replace(source, regex, fn):
  replacements = {
      m.group(0): fn(m)
      for m in regex.finditer(source)
  }
  for old, new in replacements.items():
    source = source.replace(old, new)
  return source


def normalize_inline_code_block(source):
  code_block_re = re.compile(r'''
      ^((?:{:\s*[^}]+})?```)  # group 1: initial ``` OR {:.class}```
      ((?:(?!(?:```$)).)*?)   # group 2: everything until \n```
      \n?```$                 # ending ```
  ''', re.MULTILINE | re.DOTALL | re.VERBOSE)

  # Normalize inline code block finish backticks into next line, example:
  # ```
  # {% github_sample path/to/file tag:tag % }```
  return code_block_re.sub(r'\1\2\n```', source)


def jekyll_liquid_capture(source):
  jekyll_liquid_capture_re = re.compile(r'''
      {%\s*capture\s+(?P<name>\w+)\s*%}         # {% capture variable_name %}
      (?P<body>(?:(?!{%\s*endcapture\s*%}).)*)  # Everything until endcapture
      {%\s*endcapture\s*%}                      # {% endcapture %}
  ''', re.VERBOSE | re.DOTALL)

  def fn(m):
    return (
        '{{% set {} %}}'
        '{}'
        '{{% endset %}}'
    ).format(m['name'], m['body'])
  return replace(source, jekyll_liquid_capture_re, fn)


def jekyll_liquid_include(source):
  jekyll_liquid_include_re = re.compile(r'''
      {%\s*include\s+
      (?P<file>"[^"]*"|[^\s]+)\s+                     # include_file
      (?P<args>(?:\w+\s*=\s*(?:"[^"]*"|[^\s]+)\s+)*)  # arg="text" arg=variable
      \s*%}
  ''', re.VERBOSE)

  jekyll_liquid_include_arg_re = re.compile(r'''
      (?P<name>\w+)\s*=\s*(?P<value>"[^"]*"|[^\s]+)   # arg="text" arg=variable
  ''', re.VERBOSE)

  def fn(m):
    include_file = m['file'].strip('\'"')
    args = [
        '"{}": {}'.format(arg['name'], arg['value'])
        for arg in jekyll_liquid_include_arg_re.finditer(m['args'])
    ]
    return (
        '{{% with include={{{}}} %}}'
        '{{% include "{}" %}}'
        '{{% endwith %}}'
    ).format(', '.join(args), include_file)
  return replace(source, jekyll_liquid_include_re, fn)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import json
import nbformat
import unittest

from . import new_notebook

source_file = 'test/hello.md'
variables_file = 'test/hello-variables.json'

with open(variables_file) as f:
  variables = json.load(f)


def md_cell(source, id=''):
  return nbformat.v4.new_markdown_cell(source, metadata={'id': id})


def code_cell(source, id=''):
  return nbformat.v4.new_code_cell(source, metadata={'id': id})


def nb_metadata(name):
  return {
      'colab': {
          'name': name,
          'toc_visible': True,
      },
      'kernelspec': {
          'name': 'python3',
          'display_name': 'python3',
      },
  }


def as_dict(nb):
  return json.loads(nbformat.writes(nb))

class NewNotebookTest(unittest.TestCase):
  def test_new_notebook_hello_world(self):
    expected = as_dict(nbformat.v4.new_notebook(
        cells=[
            md_cell('\n'.join([
                '# ' + variables['title'],
                '',
                'Hello {}!'.format(variables['name']),
            ]), 'hello-world'),
            code_cell('\n'.join([
                "# Hello world in Python.",
                "print('Hello from Python!')",
            ]), 'hello-world-code'),
        ],
        metadata=nb_metadata(variables['title']),
    ))
    actual = as_dict(new_notebook(source_file, variables))
    self.maxDiff = None
    self.assertEqual(expected, actual)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import jinja2
import json
import unittest

from io import StringIO
from unittest.mock import patch

from . import lines

source_file = 'test/hello.md'
expected_file = 'test/hello-expected.md'
variables_file = 'test/hello-variables.json'

with open(source_file) as f:
  source = f.read().rstrip()
with open(expected_file) as f:
  expected = f.read().rstrip()
with open(variables_file) as f:
  variables = json.load(f)


class ReadLinesTest(unittest.TestCase):
  def test_from_iterable(self):
    actual = '\n'.join(lines(source.splitlines(), variables))
    self.assertEqual(expected, actual)

  def test_from_file(self):
    actual = '\n'.join(lines(source_file, variables))
    self.assertEqual(expected, actual)

  @patch("sys.stdin", StringIO(source))
  def test_from_stdin(self):
    actual = '\n'.join(lines(variables=variables))
    self.assertEqual(expected, actual)

  def test_no_variables(self):
    actual = '\n'.join(lines(
      source
        .replace('{{ title }}', variables['title'])
        .replace('{{name}}', variables['name'])
        .splitlines()
    ))
    self.assertEqual(expected, actual)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import nbformat
import unittest

from . import view_the_docs


def md_cell(source, id=''):
  return nbformat.v4.new_markdown_cell(source, metadata={'id': id})


def code_cell(source, id=''):
  return nbformat.v4.new_code_cell(source, metadata={'id': id})


class ViewTheDocsTest(unittest.TestCase):
  def test_view_the_docs(self):
    expected = [
        md_cell(
            '<table align="left">'
              '<td>'
                '<a target="_blank" href="www.docs-url.com">'
                  'View the docs'
                '</a>'
              '</td>'
            '</table>',
            id='view-the-docs-top',
        ),
        md_cell('content', id='H1'),
        md_cell(
            '<table align="left">'
              '<td>'
                '<a target="_blank" href="www.docs-url.com">'
                  'View the docs'
                '</a>'
              '</td>'
            '</table>',
            id='view-the-docs-bottom',
        ),
    ]
    actual = list(view_the_docs(
        cells=[md_cell('content', id='H1')],
        docs_url='www.docs-url.com',
    ))
    self.assertEqual(expected, actual)

  def test_view_the_docs_logo(self):
    expected = [
        md_cell(
            '<table align="left">'
              '<td>'
                '<a target="_blank" href="www.docs-url.com">'
                  '<img src="www.docs-url.com/logo.png" width="32" height="32" />'
                  'View the docs'
                '</a>'
              '</td>'
            '</table>',
            id='view-the-docs-top',
        ),
        md_cell('content', id='H1'),
        md_cell(
            '<table align="left">'
              '<td>'
                '<a target="_blank" href="www.docs-url.com">'
                  '<img src="www.docs-url.com/logo.png" width="32" height="32" />'
                  'View the docs'
                '</a>'
              '</td>'
            '</table>',
            id='view-the-docs-bottom',
        ),
    ]
    actual = list(view_the_docs(
        cells=[md_cell('content', id='H1')],
        docs_url='www.docs-url.com',
        docs_logo_url='www.docs-url.com/logo.png',
    ))
    self.assertEqual(expected, actual)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import unittest

from . import sections


class SectionsTest(unittest.TestCase):
  def test_sections(self):
    expected = [
        [
            '# H1',
            'line 1\nline 2',
            '```\ncode 1\ncode 2\n```',
            'line 3',
        ],
        [
            '## H2',
            'line 4',
            '```\ncode 3\n```',
        ],
    ]
    actual = list(sections([
        'line 0',
        '# H1',
        'line 1\nline 2',
        '```\ncode 1\ncode 2\n```',
        'line 3',
        '## H2',
        'line 4',
        '```\ncode 3\n```',
    ]))
    self.assertEqual(expected, actual)

  def test_sections_start_on_header(self):
    expected = [['line 0'], ['# H1', 'line 1\nline 2']]
    actual = list(sections(
        input_file=[
            'line 0',
            '# H1',
            'line 1\nline 2',
        ],
        start_on_header=False,
    ))
    self.assertEqual(expected, actual)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from md2ipynb.util import class_re

SHELLS = {'sh', 'bash'}


def filter_classes(paragraphs, keep_classes=None):
  if keep_classes is None:
    keep_classes = {'language-py'}
    keep_classes.update({'shell-' + shell for shell in SHELLS})
  elif isinstance(keep_classes, str):
    keep_classes = {keep_classes}
  else:
    keep_classes = set(keep_classes)

  for paragraph in paragraphs:
    # Check for a paragraph class '{: .class}'.
    m = class_re.search(paragraph)
    if m:
      paragraph_class = m.group(1)
      if paragraph_class not in keep_classes:
        continue
      paragraph = paragraph.replace(m.group(), '').strip('\n')

    # Check for a code block '```class' both as 'class' and 'language-class'.
    if paragraph.startswith('```') and paragraph.endswith('```'):
      lines = paragraph.splitlines()
      lang = lines[0].lstrip('`').strip()
      possible_classes = {lang, 'language-' + lang, 'shell-' + lang}
      if lang and not keep_classes.intersection(possible_classes):
        continue
      if lang in SHELLS:
        for i in range(1, len(lines)-1):
          lines[i] = '!' + lines[i]
        paragraph = '\n'.join(lines)

    # If we're still here and the paragraph is not empty, yield it.
    if paragraph:
      yield paragraph



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import unittest

from . import apply


def lowercase(lines):
  for line in lines:
    yield line.lower()


def remove(lines, text='Hello'):
  for line in lines:
    yield line.replace(text, '*')


class ApplyTest(unittest.TestCase):
  def test_apply_empty(self):
    expected = []
    actual = apply([])
    self.assertEqual(expected, actual)

  def test_apply_list(self):
    expected = ['a', 'b', 'c']
    actual = apply(['a', 'b', 'c'])
    self.assertEqual(expected, actual)

  def test_apply_iterable(self):
    def get_inputs():
      yield 'a'
      yield 'b'
      yield 'c'
    expected = ['a', 'b', 'c']
    actual = apply(get_inputs())
    self.assertEqual(expected, actual)

  def test_apply(self):
    expected = ['* world', 'say hello']
    actual = list(apply(
        inputs=['Hello World', 'Say hello'],
        steps=[remove, lowercase],
    ))
    self.assertEqual(expected, actual)

  def test_apply_fn_args(self):
    expected = ['* world', 'say *']
    actual = list(apply(
        inputs=['Hello World', 'Say hello'],
        steps=[lowercase, (remove, 'hello')],
    ))
    self.assertEqual(expected, actual)



#!/usr/bin/env python

# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import argparse

import nbformat
import os

from . import new_notebook


def main(argv=None):
  parser = argparse.ArgumentParser()

  # Required arguments.
  parser.add_argument('input_file', help='Path to the markdown file to convert')

  # Optional arguments.
  parser.add_argument(
      '-o', '--output_file',
      help='Path of the output notebook to write.',
  )

  parser.add_argument(
      '--var',
      type=lambda value: value.split('=', 1),
      nargs='+',
      help='Sets a variable in the format "name=value".',
  )

  parser.add_argument(
      '--imports',
      type=lambda value: value.split(':', 1),
      nargs='+',
      help='File(s) to import at a certain section index. '
           'Sections are delimited by headers. '
           'Negative indices start from the last section. '
           'Must be in the format "path/to/file.md:index". '
           'Examples: "templates/setup.md:0" "templates/cleanup.md:-1"',
  )

  parser.add_argument(
      '--include-dir',
      default='.',
      help='Path to look for {% include file %} templates.',
  )

  parser.add_argument(
      '--keep-classes',
      nargs='+',
      help='Paragraph classes to keep, any other classes will be filtered out. '
           'Paragraphs without any class are always kept. '
           "Format of a class is '{: .className}',"
           'which can be at the beginning or the end of the paragraph, '
           'or a line before or after.',
  )

  parser.add_argument(
      '--notebook-title',
      help='Notebook title to show on Colab.',
  )

  parser.add_argument(
      '--docs-url',
      help='URL to the source docs page for the "View the Docs" button.',
  )

  parser.add_argument(
      '--docs-logo-url',
      help='URL to the logo to be shown for the "View the Docs" button.',
  )

  parser.add_argument(
      '--github-ipynb-url',
      help='GitHub URL of the ipynb file for the "Open in Colab" button.',
  )

  parser.add_argument(
      '--kernel',
      default='python3',
      help='Notebook kernel to use, defaults to "python3".',
  )

  args = parser.parse_args()

  try:
    variables = dict(args.var or [])
  except ValueError:
    parser.error('variables must be in the format "name:value", '
                 'use --help for more information.')

  try:
    imports = {}
    for path, index in args.imports or []:
      index = int(index)
      if index not in imports:
        imports[index] = []
      imports[index].append(path)
  except ValueError:
    parser.error('imports must be in the format "path/to/file.md:index", '
                 'use --help for more information.')

  notebook = new_notebook(
      args.input_file,
      variables=variables,
      imports=imports,
      include_dir=args.include_dir,
      keep_classes=args.keep_classes,
      notebook_title=args.notebook_title,
      docs_url=args.docs_url,
      docs_logo_url=args.docs_logo_url,
      github_ipynb_url=args.github_ipynb_url,
      kernel=args.kernel,
  )

  if args.output_file:
    output_dir = os.path.dirname(args.output_file)
    if not os.path.exists(output_dir):
      os.makedirs(output_dir)
    with open(args.output_file, 'w') as f:
      nbformat.write(notebook, f)
  else:
    print(nbformat.writes(notebook))


if __name__ == '__main__':
  main()



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import unittest

import md2ipynb
from md2ipynb import testing

from . import filter_classes


def get_paragraphs(class_names):
  def get_paragraphs_fn(test_file):
    sections = md2ipynb.read.sections(test_file)
    return md2ipynb.apply(sections, [
        md2ipynb.steps.flatten,
        (filter_classes, class_names),
    ])
  return get_paragraphs_fn


class FilterClassesTest(unittest.TestCase):
  def test_classes_code_block_A(self):
    expected, actual = testing.compare_files(
        'test/classes-code-block.md',
        'test/classes-A-code-block-expected.md',
        get_paragraphs('classA'),
    )
    self.assertEqual(expected, actual)

  def test_classes_code_block_A_language(self):
    expected, actual = testing.compare_files(
        'test/classes-code-block.md',
        'test/classes-A-code-block-expected.md',
        get_paragraphs('language-classA'),
    )
    self.assertEqual(expected, actual)

  def test_classes_code_block_B(self):
    expected, actual = testing.compare_files(
        'test/classes-code-block.md',
        'test/classes-B-code-block-expected.md',
        get_paragraphs('classB'),
    )
    self.assertEqual(expected, actual)

  def test_classes_code_block_B_language(self):
    expected, actual = testing.compare_files(
        'test/classes-code-block.md',
        'test/classes-B-code-block-expected.md',
        get_paragraphs('language-classB'),
    )
    self.assertEqual(expected, actual)

  def test_classes_code_block_AB(self):
    expected, actual = testing.compare_files(
        'test/classes-code-block.md',
        'test/classes-AB-code-block-expected.md',
        get_paragraphs(['classA', 'classB']),
    )
    self.assertEqual(expected, actual)

  def test_classes_code_block_AB_language(self):
    expected, actual = testing.compare_files(
        'test/classes-code-block.md',
        'test/classes-AB-code-block-expected.md',
        get_paragraphs(['language-classA', 'language-classB']),
    )
    self.assertEqual(expected, actual)

  def test_classes_leading_A(self):
    expected, actual = testing.compare_files(
        'test/classes-leading.md',
        'test/classes-A-expected.md',
        get_paragraphs('classA'),
    )
    self.assertEqual(expected, actual)

  def test_classes_leading_B(self):
    expected, actual = testing.compare_files(
        'test/classes-leading.md',
        'test/classes-B-expected.md',
        get_paragraphs('classB'),
    )
    self.assertEqual(expected, actual)

  def test_classes_leading_AB(self):
    expected, actual = testing.compare_files(
        'test/classes-leading.md',
        'test/classes-AB-expected.md',
        get_paragraphs(['classA', 'classB']),
    )
    self.assertEqual(expected, actual)

  def test_classes_leading_line_A(self):
    expected, actual = testing.compare_files(
        'test/classes-leading-line.md',
        'test/classes-A-expected.md',
        get_paragraphs('classA'),
    )
    self.assertEqual(expected, actual)

  def test_classes_leading_line_B(self):
    expected, actual = testing.compare_files(
        'test/classes-leading-line.md',
        'test/classes-B-expected.md',
        get_paragraphs('classB'),
    )
    self.assertEqual(expected, actual)

  def test_classes_leading_line_AB(self):
    expected, actual = testing.compare_files(
        'test/classes-leading-line.md',
        'test/classes-AB-expected.md',
        get_paragraphs(['classA', 'classB']),
    )
    self.assertEqual(expected, actual)

  def test_classes_trailing_A(self):
    expected, actual = testing.compare_files(
        'test/classes-trailing.md',
        'test/classes-A-expected.md',
        get_paragraphs('classA'),
    )
    self.assertEqual(expected, actual)

  def test_classes_trailing_B(self):
    expected, actual = testing.compare_files(
        'test/classes-trailing.md',
        'test/classes-B-expected.md',
        get_paragraphs('classB'),
    )
    self.assertEqual(expected, actual)

  def test_classes_trailing_AB(self):
    expected, actual = testing.compare_files(
        'test/classes-trailing.md',
        'test/classes-AB-expected.md',
        get_paragraphs(['classA', 'classB']),
    )
    self.assertEqual(expected, actual)

  def test_classes_trailing_line_A(self):
    expected, actual = testing.compare_files(
        'test/classes-trailing-line.md',
        'test/classes-A-expected.md',
        get_paragraphs('classA'),
    )
    self.assertEqual(expected, actual)

  def test_classes_trailing_line_B(self):
    expected, actual = testing.compare_files(
        'test/classes-trailing-line.md',
        'test/classes-B-expected.md',
        get_paragraphs('classB'),
    )
    self.assertEqual(expected, actual)

  def test_classes_trailing_line_AB(self):
    expected, actual = testing.compare_files(
        'test/classes-trailing-line.md',
        'test/classes-AB-expected.md',
        get_paragraphs(['classA', 'classB']),
    )
    self.assertEqual(expected, actual)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.


def apply(inputs, steps=None):
  while steps:
    step = steps.pop(0)
    if callable(step):
      fn = step
      args = []
    else:
      fn, *args = step
    inputs = fn(inputs, *args)
  return list(inputs)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import nbformat


def open_in_colab(cells, ipynb_github_url=None, **kwargs):
  if ipynb_github_url:
    if ipynb_github_url.startswith('https://'):
      ipynb_github_url = ipynb_github_url[len('https://'):]
    if ipynb_github_url.startswith('github.com/'):
      ipynb_github_url = ipynb_github_url[len('github.com/'):]
    yield nbformat.v4.new_markdown_cell(
        '<a href="https://colab.research.google.com/github/{}" target="_parent">'
          '<img src="https://colab.research.google.com/assets/colab-badge.svg" alt="Open in Colab"/>'
        '</a>'.format(ipynb_github_url),
        metadata={'id': 'view-in-github'},
    )

  for cell in cells:
    yield cell



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import fileinput
import jinja2
import tempfile
from io import StringIO

from . import MarkdownLoader
from . import GithubSampleExt


def lines(input_file='-', variables=None, include_dir=None, jinja_env=None):
  if not jinja_env:
    jinja_env = jinja2.Environment(
        loader=MarkdownLoader(include_dir or '.'),
        extensions=[GithubSampleExt],
    )

  # Read from a file, stdin or an iterable without any trailing newlines.
  if isinstance(input_file, str):
    # If input_file is '-', fileinput.input() will read from stdin.
    # Otherwise, it will open the file path.
    lines = [line.rstrip() for line in fileinput.input(input_file)]
  else:
    lines = [line.rstrip() for line in input_file]

  # jinja_env.from_string() doesn't apply the MarkdownLoader,
  # so we create a named temporary file instead.
  with tempfile.NamedTemporaryFile('w') as f:
    f.write('\n'.join(lines))
    f.seek(0)
    input_template = jinja_env.get_template(f.name)

  # Render the template and yield the lines.
  source = input_template.render(variables or {})
  for line in source.splitlines():
    yield line.rstrip()



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import jinja2
import unittest

from . import MarkdownLoader
from . import GithubSampleExt
from .github_sample import extract_snippet

title = 'Github sample'


class GithubSampleExtTest(unittest.TestCase):
  def test_github_sample(self):
    env = jinja2.Environment(loader=MarkdownLoader(), extensions=[GithubSampleExt])
    template = env.from_string('\n'.join([
        '# Github sample',
        '```',
        '{% github_sample /davidcavazos/md2ipynb/blob/master/examples/code/hello-world.py tag:hello_world %}',
        '```',
    ]))
    expected = '\n'.join([
        '# Github sample',
        '```',
        "# Hello world in Python.",
        "print('Hello from Python!')",
        '```',
    ])
    actual = template.render(title=title)
    self.assertEqual(expected, actual)


class ExtractSnippetTest(unittest.TestCase):
  def test_extract_snippet(self):
    expected = "print('Hello')"
    actual = extract_snippet(
        source='\n'.join([
            '# Extract snippet',
            '[START region_tag]',
            "print('Hello')",
            '[END region_tag]',
        ]),
        tag='region_tag',
    )
    self.assertEqual(expected, actual)

  def test_extract_snippet_indented(self):
    expected = '\n'.join([
        "print('Hello')",
        'if True:',
        "  print('world!')",
    ])
    actual = extract_snippet(
        source='\n'.join([
            '# Extract snippet indented',
            'def say_hello():',
            '  [START region_tag]',
            "  print('Hello')",
            '  if True:',
            "    print('world!')",
            '  [END region_tag]',
        ]),
        tag='region_tag',
    )
    self.assertEqual(expected, actual)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import jinja2
import md2ipynb
import nbformat


def new_notebook(
    input_file,
    variables=None,
    imports=None,
    notebook_title=None,
    keep_classes=None,
    shell=None,
    docs_url=None,
    docs_logo_url=None,
    github_ipynb_url=None,
    kernel='python3',
    steps=None,
    include_dir=None,
    jinja_env=None,
):

  sections = md2ipynb.read.sections(input_file, variables, include_dir, jinja_env)
  paragraphs = md2ipynb.apply(sections, [
      (md2ipynb.steps.imports, imports, variables, include_dir, jinja_env),
      md2ipynb.steps.flatten,
      (md2ipynb.steps.filter_classes, keep_classes)
  ])
  paragraphs = md2ipynb.apply(paragraphs, steps)
  cells = list(md2ipynb.apply(paragraphs, [
      md2ipynb.steps.paragraphs_to_cells,
      (md2ipynb.steps.view_the_docs, docs_url, docs_logo_url),
      (md2ipynb.steps.open_in_colab, github_ipynb_url),
  ]))

  for cell in cells:
    if notebook_title:
      break
    first_line = cell.source.splitlines()[0]
    if first_line.startswith('#'):
      notebook_title = first_line.strip('# ')

  # Create the notebook with all the cells.
  metadata = {
    'colab': {"toc_visible": True},
    'kernelspec': {'name': kernel, 'display_name': kernel},
  }
  if notebook_title:
    metadata['colab']['name'] = notebook_title

  return nbformat.v4.new_notebook(cells=cells, metadata=metadata)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import nbformat
import unittest

from . import paragraphs_to_cells


def md_cell(source, id='', metadata=None):
  cell_metadata = {'id': id}
  if metadata:
    cell_metadata.update(metadata)
  return nbformat.v4.new_markdown_cell(source, metadata=cell_metadata)


def code_cell(source, id='', metadata=None):
  cell_metadata = {'id': id}
  if metadata:
    cell_metadata.update(metadata)
  return nbformat.v4.new_code_cell(source, metadata=cell_metadata)


class ParagraphsToCellsTest(unittest.TestCase):
  def test_no_lines(self):
    expected = [
        code_cell('', '_-code'),
    ]
    actual = list(paragraphs_to_cells([
        '',
        '```\n```',
    ]))
    self.assertEqual(expected, actual)

  def test_one_line(self):
    expected = [
        md_cell('line 1', '_'),
        code_cell('code 1', '_-code'),
    ]
    actual = list(paragraphs_to_cells([
        'line 1',
        '```\ncode 1\n```',
    ]))
    self.assertEqual(expected, actual)

  def test_three_lines(self):
    expected = [
        md_cell('line 1\nline 2\nline 3', '_'),
        code_cell('code 1\ncode 2\ncode 3', '_-code'),
    ]
    actual = list(paragraphs_to_cells([
        'line 1\nline 2\nline 3',
        '```\ncode 1\ncode 2\ncode 3\n```',
    ]))
    self.assertEqual(expected, actual)

  def test_cell_id_with_headers(self):
    expected = [
        md_cell('line 0', '_'),
        code_cell('code 0', '_-code'),
        md_cell('# H1\n\nline 1', 'h1'),
        code_cell('code 1', 'h1-code'),
        md_cell('line 2', 'h1-2'),
        md_cell('## H2 & sym !@#$%^&*()_+~`- bols', 'h2-sym-_-bols'),
        code_cell('code 2', 'h2-sym-_-bols-code'),
        md_cell('line 3', 'h2-sym-_-bols-2'),
        code_cell('code 3', 'h2-sym-_-bols-code-2'),
        code_cell('code 4', 'h2-sym-_-bols-code-3'),
    ]
    actual = list(paragraphs_to_cells([
        'line 0',
        '```\ncode 0\n```',
        '# H1',
        'line 1',
        '```\ncode 1\n```',
        'line 2',
        '## H2 & sym !@#$%^&*()_+~`- bols',
        '```\ncode 2\n```',
        'line 3',
        '```\ncode 3\n```',
        '```\ncode 4\n```',
    ]))
    self.assertEqual(expected, actual)

  def test_code_cell_form_view(self):
    expected = [
        code_cell('code', '_-code'),
        code_cell('#@title Title', '_-code-2', {'cellView': 'form'}),
        code_cell('x = 42 #@param', '_-code-3', {'cellView': 'form'}),
    ]
    actual = list(paragraphs_to_cells([
        '```\ncode\n```',
        '```\n#@title Title\n```',
        '```\nx = 42 #@param\n```',
    ]))
    self.assertEqual(expected, actual)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import unittest

from . import imports


class ImportsTest(unittest.TestCase):
  def test_imports_at_start(self):
    expected = [
        ['# title', 'start'],
        ['# subtitle'],
        ['# section 1'],
        ['# section 1.1'],
        ['# section 2'],
    ]
    actual = list(imports(
        sections=[['# section 1'], ['# section 2']],
        imports={
            0: [['# title', 'start', '# subtitle']],
            1: [['# section 1.1']],
        },
    ))
    self.assertEqual(expected, actual)

  def test_imports_at_end(self):
    expected = [
        ['# section 1'],
        ['# section 1.1'],
        ['# section 2'],
        ['# pre-end', ':)'],
        ['# end'],
    ]
    actual = list(imports(
        sections=[['# section 1'], ['# section 2']],
        imports={
            -1: [['# pre-end', ':)', '# end']],
            -2: [['# section 1.1']],
        },
    ))
    self.assertEqual(expected, actual)

  def test_imports_file(self):
    expected = [
        ['# Title'],
        ['# section 1'],
        ['# section 2'],
        ['## Clean up', "You're all done 🎉🎉"]
    ]
    actual = list(imports(
        sections=[['# section 1'], ['# section 2']],
        imports={
            0: ['test/title.md'],
            -1: ['test/cleanup.md'],
        },
        variables={'title': 'Title'},
    ))
    self.assertEqual(expected, actual)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import unittest

from md2ipynb import testing

from . import paragraphs


class ParagraphsTest(unittest.TestCase):
  def test_paragraphs(self):
    expected, actual = testing.compare_files(
        'test/paragraphs.md',
        'test/paragraphs-expected.md',
        paragraphs)
    self.assertEqual(expected, actual)

  @unittest.skip('TODO: fix html2md to not replace multiple newlines inside code blocks')
  def test_paragraphs_spaced(self):
    expected, actual = testing.compare_files(
        'test/paragraphs-spaced.md',
        'test/paragraphs-spaced-expected.md',
        paragraphs)
    self.assertEqual(expected, actual)

  def test_paragraph_classes_leading(self):
    expected, actual = testing.compare_files(
        'test/classes-leading.md',
        'test/classes-leading.md',
        paragraphs)
    self.assertEqual(expected, actual)

  def test_paragraph_classes_leading_line(self):
    expected, actual = testing.compare_files(
        'test/classes-leading-line.md',
        'test/classes-leading-line.md',
        paragraphs)
    self.assertEqual(expected, actual)

  def test_paragraph_classes_trailing(self):
    expected, actual = testing.compare_files(
        'test/classes-trailing.md',
        'test/classes-trailing.md',
        paragraphs)
    self.assertEqual(expected, actual)

  def test_paragraph_classes_trailing_line(self):
    expected, actual = testing.compare_files(
        'test/classes-trailing-line.md',
        'test/classes-trailing-line.md',
        paragraphs)
    self.assertEqual(expected, actual)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from .flatten import flatten
from .filter_classes import filter_classes
from .imports import imports
from .open_in_colab import open_in_colab
from .paragraphs_to_cells import paragraphs_to_cells
from .view_the_docs import view_the_docs



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import re

from md2ipynb.util import class_re

from . import lines


def paragraphs(input_file='-', variables=None, include_dir=None, jinja_env=None):
  is_paragraph_done = False
  in_code_block = False
  paragraph_lines = []

  for raw_line in lines(input_file, variables, jinja_env):
    # `raw_line` is the unmodified line.
    # `line` has the paragraph class {: .class} removed if any.
    line = class_re.sub('', raw_line)

    # If a paragraph is marked as done, check for a trailing
    # paragraph class {: class} and then yield the paragraph.
    if is_paragraph_done:
      is_paragraph_done = False
      if paragraph_lines:
        trailing_paragraph_class = raw_line and not line
        if trailing_paragraph_class:
          paragraph_lines.append(raw_line)
        yield '\n'.join(paragraph_lines)
        paragraph_lines = []
        if trailing_paragraph_class:
          continue

    # Code block ends.
    if in_code_block:
      paragraph_lines.append(raw_line)
      if line.startswith('```'):
        in_code_block = False
        is_paragraph_done = True

    # Code block starts.
    elif not in_code_block and line.startswith('```'):
      in_code_block = True
      if len(paragraph_lines) == 1 and class_re.match(paragraph_lines[0]):
        paragraph_lines.append(raw_line)
      else:
        if paragraph_lines:
          yield '\n'.join(paragraph_lines)
        paragraph_lines = [raw_line]

    # Header starts.
    elif line.startswith('#'):
      if len(paragraph_lines) == 1 and class_re.match(paragraph_lines[0]):
        paragraph_lines.append(raw_line)
      else:
        if paragraph_lines:
          yield '\n'.join(paragraph_lines)
        paragraph_lines = [raw_line]
      is_paragraph_done = True

    # Non-empty lines are just appended to the current paragraph.
    elif raw_line:
      paragraph_lines.append(raw_line)

    # Empty line means the paragraph ended so yield it.
    else:
      if paragraph_lines:
        yield '\n'.join(paragraph_lines)
        paragraph_lines = []

  # Yield any remaining paragraph that wasn't triggered to yield before.
  if paragraph_lines:
    yield '\n'.join(paragraph_lines)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import nbformat
import re

invalid_cell_id_chars = re.compile(r'[^\w]+')


def paragraphs_to_cells(paragraphs):
  used_ids = set()
  def cell_id(name):
    if not name:
      return ''
    name = invalid_cell_id_chars.sub('-', name.lower()).strip('-')
    if name not in used_ids:
      used_ids.add(name)
      return name
    i = 2
    while True:
      numbered = '{}-{}'.format(name, i)
      if numbered not in used_ids:
        used_ids.add(numbered)
        return numbered
      i += 1

  last_header = '_'
  contents = []
  for paragraph in paragraphs:
    lines = paragraph.splitlines()
    if not lines:
      continue

    if lines[0].startswith('#'):
      if contents:
        yield nbformat.v4.new_markdown_cell(
            source='\n\n'.join(contents),
            metadata={'id': cell_id(last_header)},
        )
      contents = [paragraph]
      last_header = lines[0].lstrip('#').strip()
    elif lines[0].startswith('```') and lines[-1].startswith('```'):
      if contents:
        yield nbformat.v4.new_markdown_cell(
            source='\n\n'.join(contents),
            metadata={'id': cell_id(last_header)},
        )
        contents = []
      source = '\n'.join(lines[1:-1])
      metadata={'id': cell_id(last_header + '-code')}
      if '#@title' in source or '#@param' in source:
        metadata.update({'cellView': 'form'})
      yield nbformat.v4.new_code_cell(
          source=source,
          metadata=metadata,
      )
    else:
      contents.append(paragraph)
  if contents:
    yield nbformat.v4.new_markdown_cell(
        source='\n\n'.join(contents),
        metadata={'id': cell_id(last_header)},
    )


# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import setuptools

with open("README.md") as f:
  long_description = f.read()

with open('requirements.txt') as f:
  requirements = f.read().splitlines()

setuptools.setup(
    name="md2ipynb",
    version="0.1.3",
    author="David Cavazos",
    author_email="dcavazosw@gmail.com",
    description="Markdown to Jupyter Notebook converter.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/davidcavazos/md2ipynb",
    packages=setuptools.find_packages(),
    install_requires=requirements,
    test_suite='nose.collector',
    tests_require=['nose'],
    entry_points={
        'console_scripts': [
            'md2ipynb = md2ipynb.__main__:main'
        ],
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Build Tools",
        "Topic :: Software Development :: Code Generators",
        "Topic :: Software Development :: Compilers",
    ],
)


# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import re
import requests

from jinja2 import TemplateSyntaxError
from jinja2 import lexer
from jinja2 import nodes
from jinja2.ext import Extension


class GithubSampleExt(Extension):
  # Format: /<owner>/<repo>/blob/<branch>/<path>
  github_path_re = re.compile(
      r'^/([^/]+)/([^/]+)/blob/([^/]+)/([\w/.-]+)$')

  # A set of names that trigger the extension.
  tags = set(['github_sample'])

  def parse(self, parser):
    lineno = next(parser.stream).lineno

    github_path = ''
    tag = None
    last_token_type = None
    for token in parser.stream:
      if last_token_type == lexer.TOKEN_NAME and token.value == 'tag':
        parser.stream.expect(lexer.TOKEN_COLON)
        tag = parser.stream.expect(lexer.TOKEN_NAME).value
        break
      github_path += token.value
      last_token_type = token.type

    m = self.github_path_re.match(github_path)
    if not m:
      raise TemplateSyntaxError(
          'Github file path must be in the format '
          '/<owner>/<repo>/blob/<branch>/<path>, '
          'found {}'.format(repr(github_path)),
          lineno, parser.name, parser.filename)

    owner, repo, branch, path = m.groups()
    call = self.call_method('_github_sample', [
        nodes.Const(owner),
        nodes.Const(repo),
        nodes.Const(branch),
        nodes.Const(path),
        nodes.Const(tag),
    ])
    return nodes.CallBlock(call, [], [], []).set_lineno(lineno)

  def _github_sample(self, owner, repo, branch, path, tag, caller):
    url = 'https://raw.githubusercontent.com/{}/{}/{}/{}'.format(
        owner, repo, branch, path)
    req = requests.get(url, params={'ref': branch})
    assert req.status_code == requests.codes.ok, '{} {}, contents:\n{}'.format(
        req, url, req.text)
    return extract_snippet(req.text, tag)


def extract_snippet(source, tag):
  tag_start_re = re.compile(r'\[\s*START\s+{}\s*\]'.format(tag))
  tag_end_re = re.compile(r'\[\s*END\s+{}\s*\]'.format(tag))

  started = False
  min_indent = float('Inf')
  snippet = []
  for line in source.splitlines():
    if not started and tag_start_re.search(line):
      started = True
    elif started:
      if tag_end_re.search(line):
        break
      snippet.append(line)
      if line.strip():
        indent = len(line) - len(line.lstrip())
        min_indent = min(indent, min_indent)

  if min_indent != float('Inf'):
    snippet = [line[min_indent:] for line in snippet]
  return '\n'.join(snippet)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import nbformat


def view_the_docs(cells, docs_url=None, docs_logo_url=None):
  if docs_url:
    docs_logo_html = ''
    if docs_logo_url:
      docs_logo_html = \
          '<img src="{}" width="32" height="32" />'.format(docs_logo_url)
    view_the_docs_html = (
        '<table align="left">'
          '<td>'
            '<a target="_blank" href="{}">'
              '{}View the docs'
            '</a>'
          '</td>'
        '</table>'
    ).format(docs_url, docs_logo_html)

    yield nbformat.v4.new_markdown_cell(
        source=view_the_docs_html,
        metadata={'id': 'view-the-docs-top'},
    )

  for cell in cells:
    yield cell

  if docs_url:
    yield nbformat.v4.new_markdown_cell(
        source=view_the_docs_html,
        metadata={'id': 'view-the-docs-bottom'},
    )
